/*
* vim: set sw=8 ts=8 si :
* Author: Guido Socher, Copyright: GPL 2
* Chip type           : ATMEGA8
* Clock frequency     : Internal clock 8 Mhz 
*/
#include <avr/io.h>
#include <inttypes.h>
#define F_CPU 8000000UL  // 8 MHz
#include <util/delay.h>
#include <avr/interrupt.h>

// change this to calibrate the timer (max value=255 min val=100).
// A lower value means "run faster". If the internal oscillator was
// perfect then this should have the value 223:
#define CLOCKDIV 222
static volatile uint8_t runtimer=0;
static volatile uint16_t sec=0;
//Connections of the seven segment elements: 
//     10 9 7 6 4 2 1 = pin
//   0| -  f a b c d e | 0111111 | 3f
//   1| -  - - b c - - | 0001100 | 0c
//   2| g  - a b - d e | 1011011 | 5b
//   3| g  - a b c d - | 1011110 | 5e
//   4| g  f - b c - - | 1101100 | 6c
//   5| g  f a - c d - | 1110110 | 76
//   6| g  f a - c d e | 1110111 | 77
//   7| -  - a b c - - | 0011100 | 1c
//   8| g  f a b c d e | 1111111 | 7f
//   9| g  f a b c d - | 1111110 | 7e
// digit to seven segment LED mapping:
static unsigned char d2led[]={0x3f,0x0c,0x5b,0x5e,0x6c,0x76,0x77,0x1c,0x7f,0x7e};

void delay_ms(unsigned int ms)
{
        // if we would use the library function _delay_ms
        // everywhere then the code would become very fat
        // We just need integers therefore we wrap it.
	while(ms){
		_delay_ms(0.96);
		ms--;
	}
}

static volatile uint8_t cnt2step=0;

// This function gets now called in 35Hz intervals
// It is our interrupt service routine
ISR(TIMER2_COMP_vect){
        cnt2step++;
        if (cnt2step>34){
		if (runtimer){
			sec++;
			if (sec>999){
				sec=1;
			}
		}
                cnt2step=0;
        }
}

/* setup timer T2 as an interrupt generating time base.
* You must call once sei() in the main program */
void init_cnt2(void)
{
        cnt2step=0;
        TIMSK=(1<<OCIE2); // compare match on OCR2
        TCNT2=0;  // init counter
        OCR2=CLOCKDIV; // value to compare against 7812.5Hz/223=35Hz
        TCCR2=(1<<WGM21); // do not change any output pin, clear at compare match with OCR2
        // divide clock by 1024: 8MHz/1024=7812.5Hz
        TCCR2|=(1<<CS02)|(1<<CS01)|(1<<CS00); // clock divider, start counter
}



void init7segment(void)
{
	// least significant digit, cathode transistor:
        DDRD|= (1<<DDD7);
	// middle digit:
        DDRB|= (1<<DDB6);
	// left digit:
        DDRB|= (1<<DDB7);
	// digit pins:
        DDRD|= 0x7F;
	// off
	PORTD=0;
}
// Update the sevent segement display.
// You must loop over this function all the time.
inline void upd7segment(unsigned int number)
{
        // LS digit:
        PORTD=d2led[(number%10)];
        //
        PORTD|=(1<<PD7);// digit on for a short time:
        delay_ms(4);
        number/=10; // divide by 10 (shift off right most digit)
        PORTD&=~(1<<PD7);
        if (number>0){ // chop off the two left most digits if it is zero
                // middle digit:
                PORTD=d2led[(number%10)];
                //
                PORTB|= (1<<PB6);// digit on for a short time:
                delay_ms(4);
                number/=10; // divide by 10 (shift off right most digit)
                PORTB &= ~(1<<PB6);
                if (number>0){ // chop off left most digit if it is zero
                        //
                        // left digit, we assume there that the number is
                        // smaller than 999, no modulo here:
                        PORTD=d2led[number];
                        //
                        PORTB|= (1<<PB7);// digit on for a short time:
                        delay_ms(4);
                        PORTB &= ~(1<<PB7);
                }
        }
}

int main(void)
{
	unsigned char debounce=0;
        // enable PB1, as digital input
        DDRC&= ~(1<<DDC2);
        PORTC|= (1<<PINC2); // internal pullup resistor on, jumper goes to ground
        DDRC&= ~(1<<DDC1);
        PORTC|= (1<<PINC1); // internal pullup resistor on, jumper goes to ground
        DDRB&= ~(1<<DDB1);
        PORTB|= (1<<PINB1); // internal pullup resistor on, jumper goes to ground
        DDRB&= ~(1<<DDB2);
        PORTB|= (1<<PINB2); // internal pullup resistor on, jumper goes to ground
	init7segment();
        init_cnt2();
        sei();

        while (1) {
		if (debounce==0){
			if (bit_is_clear(PINC,PINC2)){
				runtimer=0;
				debounce=10; // wait polling the button to debounce
			}
			if (bit_is_clear(PINC,PINC1)){
				runtimer=1;
				debounce=10; // wait polling the button to debounce
			}
			if (bit_is_clear(PINB,PINB2)){
				sec=0;
				debounce=10; // wait polling the button to debounce
			}
			if (bit_is_clear(PINB,PINB1)){
				// toggle
				if (runtimer==1){
					runtimer=0;
				}else{
					runtimer=1;
				}
				debounce=100; // wait polling the button to debounce
			}
		}else{
			debounce--;
		}
		upd7segment(sec);
        }
	return(0);
}


